# An example of producing a predicted surface using a particular variogram
# we use the piazza road data here and a Gaussian covariance function.

source('showdat.r')

covpow = function(locs,pow=2,scale=5){
# returns a covariance matrix whose covariance terms C = ij
# are given by C = ij = exp(-[d(locs_i,locs_j)/scale]^2
# where d(a,b) gives the distance between vectors a and b
  # browser()
  d1 = dist(locs)
  n = dim(locs)[1]
  mat = matrix(0,n,n)
  mat[lower.tri(mat)] = d1
  mat = mat+t(mat)
  cc = exp(-(mat/scale)^pow)
  return(cc)
}

gaupost <- function(y,Cy,mx,Cx){
 # computes the posterior distribution of x given observed
 # data y; the data model is normal centered at x with var Cy;
 # the prior for x in N(mx,Cx)
 Sxi <- t(solve(Cx+Cy,Cx))
 mpost <- mx+Sxi%*%y
 Cpost <- Cx-Sxi%*%Cx
 return(list(mean=mpost,cov=Cpost))
}

pdat <- matrix(scan("piazza.dat"), ncol=7,byrow=T)
x <- pdat[,5]; y <- pdat[,6]; z <- log(pdat[,3])

n <- 120
nbig <- length(x); 
ok <- sample(1:nbig,n)
zr <- range(z[ok])+c(-.95,.2)

# plot for the shortcourse...
par(mfcol=c(1,3),mar=c(7,5,3,1),oma=c(5,0,0,0))
showdat(x[ok],y[ok],z[ok],zlim=zr,xlab="x",ylab="y")
xygrid <- expand.grid(seq(0,100,length=15),seq(0,200,length=30))
points(xygrid[,1],xygrid[,2],pch='.')
locs <- matrix(c(x[ok],xygrid[,1],y[ok],xygrid[,2]),ncol=2)
zdat <- c(z[ok],xygrid[,1]*0)
cx <- covpow(locs,pow=2,scale=30)*var(z[ok])
mx <- 0*zdat
vdat <- rep(100000,length(zdat))
vdat[1:n] <- .3^2
cdat <- diag(vdat)
a <- gaupost(zdat,cdat,mx,cx)
ipred <- 1:length(xygrid[,1])+n

zpred <- matrix(a$mean[ipred],nrow=length(unique(xygrid[,1])))
image(unique(xygrid[,1]),unique(xygrid[,2]),zpred,zlim=zr,xlab='x',ylab='y')
sd <- matrix(sqrt(diag(a$cov)[ipred]),nrow=length(unique(xygrid[,1])))
contour(unique(xygrid[,1]),unique(xygrid[,2]),sd)


